#include <gtkmm/scrolledwindow.h>
#include <gtkmm/label.h>
#include <gtkmm/actiongroup.h>
#include <gtkmm/filechooserdialog.h>

#include <gtkmm/viewport.h>
#include <gtkmm/adjustment.h>
#include <gtkmm/expander.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/radiobuttongroup.h>
#include <gtkmm/progressbar.h>
#include <gtkmm/main.h>

#include <cstdio>
#include <iostream>

#include "gtkpapertape.h"
using namespace Gtk;

// private start
Glib::ustring PaperTape::ui_info =
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='New'/>"
"      <menuitem action='NewFont'/>"
"      <separator/>"
"      <menuitem action='Open'/>"
"      <menuitem action='Reload'/>"
"      <menuitem action='SetNullBytes'/>"
"      <separator/>"
"      <menuitem action='Save'/>"
"      <menuitem action='SaveAs'/>"
"      <separator/>"
"      <menuitem action='ExportSVG'/>"
"      <menuitem action='ExportPNG'/>"
"      <separator/>"
"      <menuitem action='Close'/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='EditMenu'>"
"      <menuitem action='EnableEditor'/>"
"      <separator/>"
"      <menuitem action='Undo'/>"
"      <menuitem action='Redo'/>"
"      <separator/>"
"      <menuitem action='Cut'/>"
"      <menuitem action='Copy'/>"
"      <menuitem action='Paste'/>"
"      <menuitem action='Delete'/>"
"      <menuitem action='Clear'/>"
"      <separator/>"
"      <menuitem action='SelectAll'/>"
"      <menuitem action='SelectInvert'/>"
"      <menuitem action='SelectNone'/>"
"      <separator/>"
"      <menuitem action='InsertBytes'/>"
"      <menuitem action='InsertFont'/>"
"    </menu>"
"    <menu action='ViewMenu'>"
"      <menuitem action='ZoomIn'/>"
"      <menuitem action='ZoomOut'/>"
"      <separator/>"
"      <menuitem action='Zoom100'/>"
"      <menuitem action='Zoom50'/>"
"      <menuitem action='Zoom200'/>"
"      <separator/>"
"      <menuitem action='ZoomWidth'/>"
"      <menuitem action='ZoomLength'/>"
"      <menuitem action='ZoomDialog'/>"
"      <separator/>"
"      <menuitem action='Rotate90CW'/>"
"      <menuitem action='Rotate90CCW'/>"
"      <menuitem action='FlipHoriz'/>"
"      <menuitem action='FlipVert'/>"
"      <separator/>"
"      <menuitem action='SetQuality'/>"
"    </menu>"
"    <menu action='WindowMenu'>"
"      <menuitem action='NewWin'/>"
"      <separator/>"
"      <menuitem action='EnableStatusbar'/>"
"      <menuitem action='EnableEffects'/>"
"    </menu>"
"    <menu action='ColorMenu'>"
"      <menuitem action='outer_background_color'/>"
"      <menuitem action='papertape_background_color'/>"
"      <menuitem action='one_code_hole_color'/>"
"      <menuitem action='zero_code_hole_color'/>"
"      <menuitem action='feed_hole_color'/>"
"      <menuitem action='highlight_region_color'/>"
"      <menuitem action='highlight_row_color'/>"
"      <menuitem action='highlight_bit_color'/>"
"      <separator/>"
"      <menuitem action='outer_background_toggle'/>"
"      <menuitem action='papertape_background_toggle'/>"
"      <menuitem action='one_code_hole_toggle'/>"
"      <menuitem action='zero_code_hole_toggle'/>"
"      <menuitem action='feed_hole_toggle'/>"
"    </menu>"
"    <menu action='HelpMenu'>"
"      <menuitem action='About'/>"
"    </menu>"
"  </menubar>"
"  <toolbar name='ToolBar'>"
"    <toolitem action='New'/>"
"    <toolitem action='NewFont'/>"
"    <toolitem action='Open'/>"
"    <separator/>"
"    <toolitem action='ZoomIn'/>"
"    <toolitem action='ZoomOut'/>"
"    <separator expand='true'/>"
"    <toolitem action='EnableEditor'/>"
"  </toolbar>"
"  <popup name='PopUp'>"
"      <menuitem action='Cut'/>"
"      <menuitem action='Copy'/>"
"      <menuitem action='Paste'/>"
"      <menuitem action='Clear'/>"
"      <menuitem action='Delete'/>"
"      <menuitem action='InsertBytes'/>"
"  </popup>"
"</ui>";


/** Now the ActionEntries.
    gtkmm has no interface for Gtk::ActionGroup->add_actions(GtkActionEntry*,int,...),
    so we'll use the plain GTK+ functions to use this practical feature: */
/* { name, stock_id, label, acclerator, tooltip, callback } */
GtkActionEntry PaperTape::ui_common_entries[] = {
	{ "NewWin", NULL, "New _Window", "<Ctrl>n", "Open new papertape window" },
	{ "New", GTK_STOCK_NEW, "New File", NULL, "Start with a new empty file" },
	{ "NewFont", NULL, "New Label", NULL, "Start with a new empty file and start typing a label" },
	{ "Open", GTK_STOCK_OPEN, NULL, "<Ctrl>o", "Open and display an existing binary file" },
	{ "Reload", GTK_STOCK_REVERT_TO_SAVED, NULL, NULL, "Reload file from disk. Discard ALL changes" },
	{ "SetNullBytes", NULL, "Set number of null bytes", NULL, "Set number of virtual Empty bytes to display at start and end " },
	{ "Quit", GTK_STOCK_QUIT },
/*	
	{ "ZoomIn", GTK_STOCK_ZOOM_IN },
	{ "ZoomOut", GTK_STOCK_ZOOM_OUT },
	{ "ZoomReal", GTK_STOCK_ZOOM_100 },
	{ "SetZoom", GTK_STOCK_PROPERTIES, "Set Zoom level", "<ctrl>z" },

	{ "RotateClockwise", GTK_STOCK_ORIENTATION_LANDSCAPE, "Rotate clockwise" },
	{ "RotateAntiClockwise", GTK_STOCK_ORIENTATION_REVERSE_LANDSCAPE, "Rotate anticlockwise" },
	{ "MirrorHorizontally", GTK_STOCK_ORIENTATION_PORTRAIT, "Flip horizontally" },
	{ "MirrorVertically", GTK_STOCK_ORIENTATION_REVERSE_PORTRAIT, "Flip verticall" },
*/	
	{ "SetQuality", NULL, "Set display rendering quality" },
	{ "About", GTK_STOCK_ABOUT },
	
	// a bit strange: Needing actions for the menus:
	{ "FileMenu", GTK_STOCK_NEW, "_File" },
	{ "EditMenu", NULL, "_Edit" },
	{ "ViewMenu", NULL, "_View" },
	{ "ColorMenu", NULL, "_Colors" },
	{ "WindowMenu", NULL, "_Windows" },
	{ "HelpMenu", NULL, "_Help" }
};

GtkActionEntry PaperTape::ui_file_entries[] = {
	{ "Save", GTK_STOCK_SAVE },
	{ "SaveAs", GTK_STOCK_SAVE_AS },
	{ "ExportSVG", NULL, "Export to PNG...", NULL, "Export complete papertape with current settings to PNG graphics" },
	{ "ExportPNG", NULL, "Export to SVG...", NULL, "Export complete papertape with current settings to SVG graphics" },
	{ "Close", GTK_STOCK_CLOSE, "Close", NULL, "Close opened file." },
};

GtkActionEntry PaperTape::ui_editor_entries[] = {
	{ "Undo", GTK_STOCK_UNDO },
	{ "Redo", GTK_STOCK_REDO },
	{ "Cut", GTK_STOCK_CUT },
	{ "Copy", GTK_STOCK_COPY },
	{ "Paste", GTK_STOCK_PASTE },
	{ "Delete", GTK_STOCK_DELETE },
	{ "Clear", NULL, "Clear", NULL, "Clear all selected bytes to Null bytes (0x00)" },
	{ "SelectAll", GTK_STOCK_SELECT_ALL },
	{ "SelectInvert", NULL, "Invert selection", NULL, "Invert current selection" },
	{ "SelectNone", NULL, "Remove selection", NULL, "Select nothing" },
	{ "InsertBytes", GTK_STOCK_ADD, "Insert blank bytes", NULL, "Insert a number of blank bytes after the current position" },
	{ "InsertFont", GTK_STOCK_ADD, "Insert label", NULL, "Insert a label (generated paper tape font)" }
};

/* { name, stock_id, label, accelerator, tooltip, callback, is_active } */
GtkToggleActionEntry PaperTape::ui_toggle_entries[] = {
	{ "EnableEditor", GTK_STOCK_EDIT, "Enable editing", NULL, "Don't treat paper tape file as read only, but change bits and bytes interactively" },
	{ "ZoomFit", GTK_STOCK_ZOOM_FIT },
	{ "EnableStatusbar", NULL, "Enable Statusbar" },
	{ "EnableEffects", NULL, "Enable Effects" },
};

// private end

void PaperTape::make_menu() {
	Glib::RefPtr<Gtk::ActionGroup> actions_common_entries = ActionGroup::create("PaperTapeCommon");
	Glib::RefPtr<Gtk::ActionGroup> actions_file_entries = ActionGroup::create("PaperTapeFile");
	Glib::RefPtr<Gtk::ActionGroup> actions_editor_entries = ActionGroup::create("PaperTapeEditor");
	Glib::RefPtr<Gtk::ActionGroup> actions_toggle_entries = ActionGroup::create("PaperTapeEditor");
	
	// this function is missing in gtkmm, therefore I must
	// use convience functions
	gtk_action_group_add_actions(
			actions_common_entries->gobj(),
			ui_common_entries,
			G_N_ELEMENTS(ui_common_entries),
			NULL);
	gtk_action_group_add_actions(
			actions_file_entries->gobj(),
			ui_file_entries,
			G_N_ELEMENTS(ui_file_entries),
			NULL);
	gtk_action_group_add_actions(
			actions_editor_entries->gobj(),
			ui_editor_entries,
			G_N_ELEMENTS(ui_editor_entries),
			NULL);
	gtk_action_group_add_toggle_actions(
			actions_toggle_entries->gobj(),
			ui_toggle_entries,
			G_N_ELEMENTS(ui_toggle_entries),
			NULL);

	this->ui_manager = UIManager::create();
	ui_manager->set_add_tearoffs(true); // we want tearoffs ;-)
	ui_manager->insert_action_group(actions_common_entries, 0);
	ui_manager->insert_action_group(actions_file_entries, 1);
	ui_manager->insert_action_group(actions_editor_entries, 2);
	ui_manager->insert_action_group(actions_toggle_entries, 3);
	
	// this is a first try, getting an action group from the View.
	ui_manager->insert_action_group(view->get_color_action_group(), 4);
	
	// and another try
	ui_manager->insert_action_group(view->zoom->get_action_group(), 5);
	
	try {
		ui_manager->add_ui_from_string(this->ui_info);
	} catch(const Glib::Error& ex) {
		std::cerr << "Gtk::PaperTape: Parsing UI Manager Menu failed: " << ex.what();
	}
	
	// connect *ALL* actions to one callback method:
	ui_manager->signal_post_activate().connect(
		sigc::mem_fun(*this, &PaperTape::menu_actions));

	//parent_window->add_accel_group(actions_common_entries);
}

void PaperTape::menu_actions(const Glib::RefPtr<Action>& action) {
	Glib::ustring name = action->get_name();
	
	if(name == "New") {
		//new_file();
	} else if(name == "NewFont") {
		std::cout << "To be implemented.";
	} else if(name == "Open") {
		model->open();
	} else if(name == "Reload") {
		model->reload();
	} else if(name == "Save") {
		model->save();
	} else if(name == "SaveAs") {
		model->saveAs();
	} else if(name == "ExportSVG" || name == "ExportPNG") {
		view->export_image();
		
		/*PaperTapeFileType target_format;
		target_format = (name == "ExportSVG") ? SVG : PNG;
		// Export file dialog.
		FileChooserDialog* dialog = new FileChooserDialog(
			*this->parent_window,
			"Select name for graphics file",
			FILE_CHOOSER_ACTION_SAVE
		);
		dialog->add_button("Export", RESPONSE_OK); // Stock::SAVE
		dialog->add_button(GTK_STOCK_CANCEL, RESPONSE_CANCEL);
		dialog->set_do_overwrite_confirmation(true);
		dialog->set_modal(true);
		dialog->set_transient_for(*this->parent_window);

		// Filters, just for the optics ;-)
		FileFilter exporter_filter;
		exporter_filter.add_pattern("*.svg");
		exporter_filter.add_pattern("*.png");
		exporter_filter.set_name("SVG and PNG files");
		dialog->add_filter(exporter_filter);

		// Make extra widget
		VBox extra_widget(FALSE, 10);
		Expander file_types_expander("Export file type: Detect by suffix"); // Dateityp: Nach Endung
		VBox file_types_box(TRUE, 2);
		RadioButtonGroup file_types_group;
		RadioButton file_types[3];
		file_types[0].set_label("Detect type automatially by suffix");
		file_types[1].set_label("Export to SVG file");
		file_types[2].set_label("Export to PNG file");
		for(int i=0; i < sizeof file_types / sizeof *file_types; i++) {
			file_types[i].set_group(file_types_group);
			file_types_box.add(file_types[i]);
		}
		file_types_expander.add(file_types_box);
		extra_widget.add(file_types_expander);

		CheckButton keep_selections("Export with current selection");
		extra_widget.add(keep_selections);
		extra_widget.show_all();

		dialog->set_extra_widget(extra_widget);

		// run it!
		if(dialog->run() == RESPONSE_CANCEL) {
			delete dialog;
			return; // canceled.
		}
		else {
			// now disable all and add a progress bar
			dialog->set_sensitive(false);
			dialog->set_response_sensitive(RESPONSE_CANCEL, true);
			
			ProgressBar* progress = new ProgressBar();
			progress->set_text("Export to ABC: 0%");
			extra_widget.add(*progress);
			extra_widget.show_all();
			std::cout << "Well, EXPORT!\n";
			
			for(int x=0; x < 100; x++) {
				progress->set_fraction(x/100.0);
				progress->set_text(g_strdup_printf("Export to ABC: %i%%", x));
				
				while( Gtk::Main::events_pending() )
					Gtk::Main::iteration();
				std::cout << "Progressing...\n";
				Glib::usleep(1000*1000);
			}
			
			delete dialog;*/

		
			/*if(view->export_file(target_format, dialog.get_filename())) {
				std::cout << "Export okay!\n";
			} else
				std::cout << "Export damaged!\n";
				*/
		//}
	} else if(name == "Close") {
		model->close();
	} else if(name == "SetNullBytes") {
	} else if(name == "Quit") {
		// quit somehow. That's problematic and complex.
		// perhaps just quit mainloop.
	} else if(name == "ZoomIn") {
		view->transform(ZOOM_IN);
	} else if(name == "ZoomOut") {
		view->transform(ZOOM_OUT);
	} else if(name == "ZoomReal") {
		view->transform(ZOOM_REAL);
	} else if(name == "SetZoom") {
		view->transform(SET_ZOOM);
	} else if(name == "RotateClockwise") {
		view->transform(ROTATE_CLOCKWISE);
	} else if(name == "RotateAntiClockwise") {
		view->transform(ROTATE_ANTICLOCKWISE);
	} else if(name == "MirrorHorizontally") {
		view->transform(MIRROR_HOR);
	} else if(name == "MirrorVertically") {
		view->transform(MIRROR_VER);
	} else if(name == "SetQuality") {
		// implementierung: wie
		// view->quality(), nach dialog. auch mit
		// enum GOOD, BAD, ANTIALIASING, WITHOUT_ALPHA,
		// etc.
	} else if(name == "About") {
		// About. We would have several about things,
		// like about
		// * The Paper Tape project in general
		// * GtkPaperTape (mm)
		// * LOCHSTREIFEN
		// * the Font generator
		// * The driver frontend, aka
		//   Paper Tape Suite
	} else if(name == "FileMenu") {
		std::cout << "Strange: Called " << name << "\n";
	} else if(name == "EditMenu") {
		std::cout << "Strange: Called " << name << "\n";
	} else if(name == "ViewMenu") {
		std::cout << "Strange: Called " << name << "\n";
	} else if(name == "ColorMenu") {
		std::cout << "Strange: Called " << name << "\n";
	} else if(name == "WindowMenu") {
		std::cout << "Strange: Called " << name << "\n";
	} else if(name == "HelpMenu") {
		std::cout << "Strange: Called " << name << "\n";
	} else if(name == "Undo") {
		std::cout << "Start Undo/redo engine for " << name << "\n";
	} else if(name == "Redo") {
		std::cout << "Start Undo/redo engine for " << name << "\n";
	} else if(name == "Cut") {
		std::cout << "Get selection and " << name << "\n";
	} else if(name == "Copy") {
		std::cout << "Get selection and " << name << "\n";
	} else if(name == "Paste") {
		std::cout << "Get cursor and " << name << "\n";
	} else if(name == "Delete") {
		std::cout << "Get selection and " << name << "\n";
	} else if(name == "Clear") {
		std::cout << "Get selection and " << name << " or clear all\n";
	} else if(name == "SelectAll") {
		std::cout << "Change selection to " << name << "\n";
	} else if(name == "SelectInvert") {
		std::cout << "Change selection to " << name << "\n";
	} else if(name == "SelectNone") {
		std::cout << "Change selection to " << name << "\n";
	} else if(name == "InsertBytes") {
		std::cout << "Get cursor and " << name << "\n";
	} else if(name == "InsertFont") {
		std::cout << "Get cursor and " << name << "\n";
	} else {
		std::cout << "Action " << action->get_name() << " not implemented!\n";
	}
}

PaperTape::PaperTape(Window& parent_window) {
	// set up nested objects
	this->parent_window = &parent_window;
	this->model = new PaperTapeFile(this);
	printf("ABC\n");
	this->view = new PaperTapeView(this);
		
	this->make_menu();

	// add menus and toolbars to our box
	Gtk::Widget* menubar = ui_manager->get_widget("/MenuBar");
	Gtk::Widget* toolbar = ui_manager->get_widget("/ToolBar");
	//this->add(*menubar);
	//this->add(*toolbar);
	this->pack_start(*menubar, PACK_SHRINK);
	this->pack_start(*toolbar, PACK_SHRINK);
	
	// set up a Scrolled Window
	ScrolledWindow* win = new ScrolledWindow();
	win->set_policy(POLICY_ALWAYS, POLICY_ALWAYS);
	
	//VBox *vbox = new VBox(TRUE, 0);
	//vbox->add(*view);
	
	win->add(*view);
	
	view->set_redraw_on_allocate(true);
		
	// set up ourself (we are a container)
	this->add(*win);
		
	/*Viewport port(
		*new Adjustment(0, 0, 100),
		*new Adjustment(0, 0, 100)
	);
	port.add(*view);
	
	this->add(port);
	*/
	this->show_all();
	
	// these two lines work:
	//this->add(*view);
	//view->show();
}
