#include "gtkpapertapecolor.h"

#include <gtkmm/box.h>
#include <gtkmm/label.h>
#include <gtkmm/menuitem.h>
#include <gtkmm/colorbutton.h>
#include <cairo/cairo.h>

#include <iostream>

using namespace Gtk;

// initialize the constant member of the class, only once in the global
// compilation process (in this object file)
//const Glib::RefPtr<SizeGroup> PaperTapeColor::label_group = SizeGroup::create(SIZE_GROUP_HORIZONTAL);

PaperTapeColor::PaperTapeColor(const Glib::ustring& name)
:	Action(name + "_color") //, Gtk::IconTheme::get_default()->get_example_icon_name(), "labelbla", "tooltipbla")
{
	std::cout << "Initialized " <<name << "\n";
}

Glib::RefPtr<PaperTapeColor> PaperTapeColor::create(
	PaperTapeView* view,
	cairo_pattern_t** target,
	const Glib::ustring& action_base_name,
	const Glib::ustring& dialog_title,
	const Glib::ustring& menu_label,
	const Glib::ustring& toggle_label
) {
	PaperTapeColor* obj = new PaperTapeColor(action_base_name);
	std::cout << "HALLO\n";
	Glib::RefPtr<PaperTapeColor> ref(obj);
	
	obj->view = view;
	obj->action_name = action_base_name;
	obj->dialog_title = dialog_title;
	obj->menu_label = menu_label;
	obj->toggle_label = toggle_label;
	obj->switchable = (toggle_label != "");
	obj->target = target;
	
	obj->property_label() = menu_label;

	// copy current target to value
	obj->value = *target;
	
	// setup toggle action
	obj->toggle_action = ToggleAction::create(
		action_base_name + "_toggle",
		toggle_label,
		"This is a toggle :-)",
		(*target != NULL)
	);
	obj->toggle_action->signal_toggled().connect(sigc::mem_fun(*obj, &PaperTapeColor::toggled));

	// setup color action
	//obj->color_action = ColorAction::create(this);
	// no more need ;-)
	
	return ref;
}

Widget* PaperTapeColor::create_menu_item_vfunc() {
	menu_item = new MenuItem();
	// the horizontal menu item box	
	HBox* menu_box = manage(new HBox(false, 4));
	// the color button on the left
	//ColorButton* button = manage(new ColorButton);
	color_button.set_use_alpha(true);
	color_button.set_title(this->dialog_title);
	menu_box->pack_start(color_button, false, false, 0);
	// the label on the right
	Label* label = manage(new Label);
	label->set_markup(this->menu_label);
	label->set_justify(JUSTIFY_CENTER);
	
	//label_group->add_widget(*label);
	
	menu_box->pack_start(*label, true, true, 0);
	// now pack'em all
	menu_item->add(*menu_box);	
	menu_item->show_all();
	
	// make clicking the menuitem activating the color chooser button
	menu_item->signal_activate().connect(sigc::mem_fun(color_button, &Gtk::Button::clicked));
	// get color setting signal
	color_button.signal_color_set().connect(sigc::mem_fun(*this, &PaperTapeColor::color_set));
	
	if(*target != NULL) {
		std::cout << "ALREADY SET COLOR FOR " << dialog_title << "\n";
		set(*target);
	} else {
		// some transparent value + disable!
		set(0., 0., 0., 0.);
		menu_item->set_sensitive(false);
	}
	return menu_item;
}

void PaperTapeColor::color_set() {
	// a color was set!
	cairo_pattern_destroy(this->value);
	Gdk::Color color = color_button.get_color();
	this->value = cairo_pattern_create_rgba(
		color.get_red_p(),
		color.get_green_p(),
		color.get_blue_p(),
		(double)color_button.get_alpha() / (double)G_MAXUINT16
	);
	std::cout << "Signal sent\n";
	color_changed.emit(this);
}

void PaperTapeColor::set(cairo_pattern_t* pattern) {
	// not very perfomant, but secure.
	double r,g,b,a;
	cairo_pattern_get_rgba(pattern, &r, &g, &b, &a);
	set(r, g, b, a);
}

void PaperTapeColor::set(double red, double green, double blue, double alpha) {
	// set color to button
	Gdk::Color color;
	color.set_rgb_p(red, green, blue);
	color_button.set_color(color);
	// save alpha, too
	color_button.set_alpha((guint16)(alpha * (double)G_MAXUINT16));
	// save color in our cairo pattern object
	cairo_pattern_destroy(this->value);
	this->value = cairo_pattern_create_rgba(red, green, blue, alpha);
	// send the color changed signal
	std::cout << "Signal sent\n";
	color_changed.emit(this);
}

void PaperTapeColor::toggled() {
	// Make Color Menuitem sensitivy dependent from toggle status
	menu_item->set_sensitive(is_displayed());
	
	if(*target == NULL)
		*target = value;
	else
		*target = NULL;

	// send the color changed signal
	std::cout << "Signal sent\n";
	color_changed.emit(this);
}

