#include "gtkpapertapeexporter.h"

#include <gtkmm/frame.h>
#include <gtkmm/messagedialog.h>

#include <iostream>
#include <cstdio>

using namespace Gtk;
using namespace Cairo;

PaperTapeExporter::PaperTapeExporter(PaperTapeView& view)
:
	FileChooserDialog(*view.controller->parent_window, "Howdy", FILE_CHOOSER_ACTION_SAVE)
{
	PaperTapeExporter(*view.controller->parent_window, view.lochstreifen);
}

PaperTapeExporter::PaperTapeExporter(Window& parent, const LOCHSTREIFEN* ls)
:
	FileChooserDialog(parent, "Select name for graphics Export", FILE_CHOOSER_ACTION_SAVE)
	,extra_widget(FALSE, 10)
{
	// COPY Lochstreifen.
	this->l = lochstreifen_copy(ls);
	
	// setup lochstreifen:
	l->row_callback = &PaperTapeExporter::update_gui;
	l->row_callback_user_data = this;
	
	add_button("Export", RESPONSE_OK);
	this->cancel_button = add_button(GTK_STOCK_CANCEL, RESPONSE_CANCEL);
	set_do_overwrite_confirmation(true);
	set_modal(true);
	set_transient_for(parent);
	printf("TEst\n");

	// Filters, just for the optics ;-)
	FileFilter* exporter_filter = manage(new FileFilter);
	exporter_filter->add_pattern("*.svg");
	exporter_filter->add_pattern("*.png");
	exporter_filter->set_name("Exportable files (*.svg, *.png)");
	printf("TEst\n");
	add_filter(*exporter_filter);
	FileFilter* all_files = manage(new FileFilter);
	all_files->set_name("All files");
	add_filter(*all_files);

	// Make extra widget
	//VBox* extra_widget = new VBox(FALSE, 10);
	Frame* file_types_expander = manage(new Frame("Export file type: Detect by suffix")); // Dateityp: Nach Endung
	VBox* file_types_box = manage(new VBox(TRUE, 2));
	RadioButtonGroup file_types_group;
	printf("TEst\n");
	type_auto_select.set_label("Detect type automatially by suffix");
	type_svg.set_label("Export to SVG file");
	type_png.set_label("Export to PNG file");
	printf("TEst\n");
	type_auto_select.set_group(file_types_group); type_svg.set_group(file_types_group); type_png.set_group(file_types_group);
	file_types_box->add(type_auto_select); file_types_box->add(type_svg); file_types_box->add(type_png);
	/*RadioButton* file_types[3] = { &type_auto_select, &type_svg, &type_png }
	for(unsigned int i=0; i < sizeof file_types / sizeof *file_types; i++) {
		file_types[i]->set_group(file_types_group);
		file_types_box->add(*file_types[i]);
	}*/
	printf("TEst\n");
	file_types_expander->add(*file_types_box);
	extra_widget.add(*file_types_expander);

		printf("TEst\n");
	CheckButton* keep_selections = manage(new CheckButton("Export with current selection"));
	extra_widget.add(*keep_selections);
	extra_widget.show_all();

	set_extra_widget(extra_widget);
	printf("TEst\n");
}

bool PaperTapeExporter::run_export() {
	if(run() == RESPONSE_CANCEL) {
		return false;
	}

	// supposed to be called after run() returned with a valid file.
	set_sensitive(false);
	// this doesn't work
	//set_response_sensitive(RESPONSE_CANCEL, true);
	// so:
	
	printf("TEst\n");
	cancel_button->set_sensitive(true); // doesn't work, too! :/
	// enable cancel action
	cancel_button->signal_clicked().connect(sigc::mem_fun(*this, &PaperTapeExporter::cancel));
	printf("TEst\n");
	this->signal_hide().connect(sigc::mem_fun(*this, &PaperTapeExporter::cancel));
	// set up cancel_export variable
	this->cancel_export = false;
	
	printf("TEst\n");
	progress.set_text("Export running...");
	extra_widget.add(progress);
	printf("TEst\n");
	extra_widget.show_all();
	
	if(type_auto_select.get_active()) {
		printf("TEst\n");
		// auto selection of file type
		try {
		if(get_filename().substr(get_filename().length()-3-1, 3) == "svg")
			type_svg.set_active(true);
		else if(get_filename().substr(get_filename().length()-3-1, 3) == "png")
			type_png.set_active(true);
		else {
			// display dialog to clear the question which file type
			// the user wants
			MessageDialog err(*this,
				"File type could not be detected",
				false, // markup
				MESSAGE_ERROR,
				BUTTONS_CANCEL,
				true);
			err.set_secondary_text("Your filename <b>"+get_filename()+"</b> "
				"does not end with a typical suffix like .png or .svg. Therefore "
				"the wanted file type could not be clearly detected. Please select one:");
			err.property_secondary_use_markup().set_value(true);
			err.add_button("PNG Image", 42);
			err.add_button("SVG Image", 43);
			switch(err.run()) {
				case 42: type_png.set_active(true); break;
				case 43: type_svg.set_active(true); break;
				default:
					// cancel
					return false;
			}
			err.hide();
		}
		} catch(...) {
			std::cout << "Caught something nasty!\n";
		}
	}

	// query width/height from LOCHSTREIFEN
	double width, height;
	lochstreifen_get_target_dimensions(l, (int*)&width, (int*)&height);

	std::cout << "HOWDY\n";
	if(type_svg.get_active()) {
		// we make a SVG picture
		// the SVG surface wants width/height in point. 1 pt = 1/72 inch
		// would be a calculation like
		// width = 72 * width / Gdk::Screen::get_default().get_resolution();

		Cairo::RefPtr<SvgSurface> surface = SvgSurface::create(get_filename(), width, height);
		Cairo::RefPtr<Context> cr = Context::create(surface);
		
		lochstreifen_draw(l, cr->cobj());
		
		// c'est ca.
		return true;
	} else if(type_png.get_active()) {
		// we make a PNG picture.
		/*
		PNG only Always produces:
			glibmm-ERROR **:
			unhandled exception (type std::exception) in signal handler:
			what: St9bad_alloc

		*/
		Cairo::RefPtr<ImageSurface> surface = ImageSurface::create(FORMAT_ARGB32, (int)width, (int)height);
		Cairo::RefPtr<Context> cr = Context::create(surface);
		
		lochstreifen_draw(l, cr->cobj());
		
		surface->write_to_png(get_filename());
		return true;
	} else {
		std::cout << "File type not detectable\n";
		return false;
	}
}

void PaperTapeExporter::update_gui(const row_t* current_row, cairo_t* context, void* user_data) {
	// yes, we must do some ugly C casting ;-)
	printf("CALLBACK! BIG CALLBACK!\n");
	PaperTapeExporter* obj = (PaperTapeExporter*) user_data;
	
	double percentage = static_cast<double>(*current_row) / (double)obj->l->data_length;
	obj->progress.set_fraction(percentage);
	// this g_strdup_printf string is never given back -- memory leck!
	// unfortunately ustring::compose doesn't compile at my site (glibmm too old?)
	obj->progress.set_text("Test");
	//get_text
	//g_strdup_printf("Exporting... %i%%", (int)(percentage*100) ));

	// make Gtk happy
	while( Gtk::Main::events_pending() )
		Gtk::Main::iteration();

	// ah, and break the whole thing if not wanted.
//	if(cancel_export)
//		return false;
	
	Glib::usleep(1000*1000); // sleep.	
}

void PaperTapeExporter::cancel() {
	// cancel a running export.
	this->cancel_export = true;
	MessageDialog info(*this,
		"Export canceled",
		false, // markup
		MESSAGE_INFO,
		BUTTONS_OK,
		true);
	info.set_secondary_text("You have cancelled the running export to the file. The picture file won't be deleted.");
	info.run();
}
