#include <gtkmm/filechooserdialog.h>
#include <gtkmm/messagedialog.h>
#include <glibmm/error.h>
#include <iostream>

#include "gtkpapertapefile.h"

using namespace Gtk;

PaperTapeFile::PaperTapeFile(PaperTape* controller) {
	this->controller = controller;

	// make some data
	data_len = 20;
	data = new byte_t[data_len];
	for(int x=0; x<data_len; x++) data[x] = x;
	
	// data have changed, fire the change thingy.
	data_changed.emit(data, data_len);

	// make a new lochstreifen.
	//lochstreifen_set_data(lochstreifen, data_len, data);
	//lochstreifen_set_scaling_by_width(lochstreifen, 200);
	//lochstreifen_print_debug(lochstreifen);
}


void PaperTapeFile::open() {
	// GUI method: Open File dialog.
	FileChooserDialog dialog(
		//controller->get_parent_window(),
		"Select (binary) file to open as a paper tape file",
		FILE_CHOOSER_ACTION_OPEN
	);
	dialog.add_button(GTK_STOCK_CANCEL, RESPONSE_CANCEL);
	dialog.add_button(GTK_STOCK_OPEN, RESPONSE_OK);

	if(controller->run_dialog(dialog) != RESPONSE_OK)
		return; // canceled.
	try {
		this->open(dialog.get_filename());
	} catch(Glib::Error::Error ex) {
		MessageDialog error(
			"Could not open file",
			false, // markup
			MESSAGE_ERROR,
			BUTTONS_OK);
		error.set_secondary_text("Could not open file <b>"+dialog.get_filename()+"</b>. "
			"The following error occured: "+ex.what());
		error.property_secondary_use_markup().set_value(true);
		controller->run_dialog(error);
	}
}

void PaperTapeFile::open(const std::string& filename) {
	//std::string Glib::file_get_contents(filename);
	GError *err = NULL;
	if(g_file_get_contents(filename.c_str(), (gchar**)&data, (gsize*)&data_len, &err) == FALSE) {
		throw Glib::Error(err, true);
	}
	std::cout << "Read in file " << filename << " successfully.\n";
	data_changed.emit(data, data_len);
}

bool PaperTapeFile::save(const std::string& filename) {
	// Save to file.
	std::cout << "FILE: Save\n";
	return true;
}

void PaperTapeFile::saveAs() {
	std::cout << "Show gui for saving as file.\n";
}

void PaperTapeFile::close() {
	std::cout << "Show gui for closing file\n";
}

void PaperTapeFile::reload() {
	std::cout << "Model: Reload file from scratch!\n";
}
