#include <cairomm/context.h>
#include <iostream>
#include <cstdio>
#include <gtkmm/colorbutton.h>

#include "gtkpapertapeview.h"
#include "gtkpapertapeexporter.h"

#include <cairo/cairo-xlib.h>
using namespace Gtk;

PaperTapeView::PaperTapeView(PaperTape* controller)
:
	just_hovered_row(LOCHSTREIFEN_NO_ROW),
	just_hovered_track(LOCHSTREIFEN_NO_TRACK),
	controller(controller)
{
	this->lochstreifen = lochstreifen_new();
	make_colors();
	
	current_width_scaling = 200;
	lochstreifen_set_scaling_by_width(lochstreifen, 200);
	
	// catch events for callback functions
	add_events(Gdk::POINTER_MOTION_MASK | Gdk::SCROLL_MASK | Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK);
	
	// the controller connects the on_data_change for us.
	controller->model->data_changed.connect(sigc::mem_fun(*this, &PaperTapeView::on_data_change));
	
	// init zoom (don't forget to delete)
	zoom = new PaperTapeZoom(0.0, Gdk::Screen::get_default()->get_resolution());
	zoom->signal_changed().connect(sigc::mem_fun(*this, &PaperTapeView::on_zoom_change));
	
	//set_double_buffered(false);
	//get_window()->set_debug_updates(true);
}

void PaperTapeView::on_data_change(byte_t* new_data, int new_data_len) {
	// daten haben sich geaendert. Neue Daten als Parameter
	// Die Daten verblieben bei dem Model, welches das hier aufruft.
	
	std::cout << "data change signal called, newl len " << new_data_len << "\n";

	lochstreifen_set_data(lochstreifen, new_data_len, new_data);
	
	set_size_request(
		lochstreifen_get_target_width(lochstreifen),
		lochstreifen_get_target_height(lochstreifen)
	);
}

void PaperTapeView::on_zoom_change() {
	// yeah, zoom has changed.
	// erhm, this is a bit stupid:
	cairo_matrix_init(&lochstreifen->matrix,
		zoom->xx, zoom->xy, zoom->yx, zoom->yy, zoom->x0, zoom->y0
	);
	lochstreifen_check_matrix(lochstreifen);	
	set_size_request(
		lochstreifen_get_target_width(lochstreifen),
		lochstreifen_get_target_height(lochstreifen)
	);
	queue_draw();
}

bool PaperTapeView::on_expose_event(GdkEventExpose *event) {
	// besser: Erst das tatsaechliche cairo_t* objekt kriegen beim drawen!
	Cairo::RefPtr<Cairo::Context> cr = this->get_window()->create_cairo_context();
	cr->save(); // this is said to be good style
	
	// --gtk-debug=updates
	// for debugging!
	
	std::cout << "expose("<< event->count <<"): " << event->area.x << "|" << event->area.y << " " <<
		event->area.width << "x" << event->area.height << "\n";

	//get_window()->clear_area(event->area.x, event->area.y, event->area.width, event->area.height);
	//gdk_window_clear_area(event->window, event->area.x, event->area.y, event->area.width, event->area.height);
	//gdk_window_clear(event->window);
	
	int x, y, width, height;
	
	gdk_window_get_position(gdk_window_get_parent(event->window), &x, &y);
	gdk_drawable_get_size(GDK_DRAWABLE(event->window), &width, &height);
	std::cout << "GdkWin: " << x << "|" << y << " " << width << "x" << height << "\n";
	//gdk_window_get_geometry(gdk_window_get_parent(event->window), &x, &y, &width, &height, NULL);
	//std::cout << x << "|" << y << " " << width << "x" << height << "\n";
	
	// Get the CAIRO WIDTH/HEIGHT
	width = cairo_xlib_surface_get_width( cairo_get_target(cr->cobj()) );
	height = cairo_xlib_surface_get_height( cairo_get_target(cr->cobj()) );
	std::cout << "csurface: " << width << "x" << height << "\n";
	
	// clipping! :-)
	lochstreifen_set_clip(lochstreifen,
		event->area.x, event->area.y,
		event->area.width, event->area.height);
	
	lochstreifen_draw(lochstreifen, cr->cobj());
	cr->restore();
	
//	cr->set_source_rgba(0.337, 0.612, 0.117, 0.9);   // green
//	cr->paint();
	
	
	return true;
}

bool PaperTapeView::on_scroll_event(GdkEventScroll* event) {
	std::cout << "Mouse Scrolling.\n";

	return true;
}

void PaperTapeView::queue_draw_bit(row_t& row, track_t& track) {
	cairo_rectangle_t repaint;
	// paint only in valid range:
	if(row != LOCHSTREIFEN_NO_ROW && track != LOCHSTREIFEN_NO_TRACK) {
		if(lochstreifen_get_target_rect_from_bit(lochstreifen, row, track, &repaint)
		   == LOCHSTREIFEN_SUCCESS) {
			std::cout << "Redrawing! row " << row << " track " << track << ": ";
			std::cout << repaint.x << "|" << repaint.y << " " << repaint.width << "x" << repaint.height << "\n";
			queue_draw_rect(repaint);
		}
		else
			std::cout << "ERROR Repainting the bit " << row << "," << track << " failed!\n";
	}
}

void PaperTapeView::queue_draw_row(row_t& row) {
	cairo_rectangle_t repaint;
	// paint only in valid range
	if(row != LOCHSTREIFEN_NO_ROW) {
		if(lochstreifen_get_target_rect_from_row(lochstreifen, row, &repaint)
		   == LOCHSTREIFEN_SUCCESS)
			queue_draw_rect(repaint);
		else	
			std::cout << "ERROR Repainting row " << row << " failed!\n";
	}
}

bool PaperTapeView::on_motion_notify_event(GdkEventMotion* event) {
	row_t new_row;
	track_t new_track;
	lochstreifen_get_target_bit_from_point(lochstreifen, &new_row, &new_track, event->x, event->y);

	// repaint bits, if neccessary
	if(new_track != just_hovered_track) {
		// we have changed the track, queue a redraw for the old one
		queue_draw_bit(just_hovered_row, just_hovered_track);
	
		// set the new one. If NO_TRACK or NO_ROW, this is the "no highlight"
		// state, internally
		lochstreifen_set_highlight_bit(lochstreifen, new_row, new_track);
		std::cout << "Queue new draw row " << new_row << " bit " << new_track << "\n";
		// redraw the new one. If NO_TRACK or NO_ROW, it won't be paint.
		queue_draw_bit(new_row, new_track);
		
		// buffer hovered track for next mouse event
		just_hovered_track = new_track;
	} // bit
	
	// repaint the old row, if neccessary
	if(new_row != just_hovered_row) {
		// we have changed the row, repaint the old one (remove the highlight)
		queue_draw_row(just_hovered_row);
	
		// if new_row is NO_ROW, this will simply remove the highlight
		lochstreifen_set_highlight_row(lochstreifen, new_row);
		// this will only repaint if it's a vaild row
		queue_draw_row(new_row);

		// buffer hovered row for next mouse event
		just_hovered_row = new_row;
	} // row

	return true;
}

bool PaperTapeView::on_button_press_event(GdkEventButton* event) {
	// mouse button pressed. (we could make another color ;-)
	if(just_hovered_track != LOCHSTREIFEN_NO_TRACK) {
		// we are on a bit, and the mouse button was pressed.
		std::cout << "You are about to change byte " << just_hovered_row << ", bit " << just_hovered_track << "\n";
		
		// make a cool surface:
		cairo_pattern_destroy(lochstreifen->highlight_bit_color);
		lochstreifen->highlight_bit_color = cairo_pattern_create_rgb(0., 1., 0.);
		
		queue_draw_bit(just_hovered_row, just_hovered_track);
	} else if(just_hovered_row != LOCHSTREIFEN_NO_ROW) {
		// mouse pressed on a row, but not on Button
		std::cout << "Begin selection\n";
	}

	return true;
}

bool PaperTapeView::on_button_release_event(GdkEventButton* event) {
	// mouse button released.
	if(just_hovered_track != LOCHSTREIFEN_NO_TRACK) {
		// that's it. CHANGE THE DATA!
		std::cout << "Triggered. Have toggled byte " << just_hovered_row << ", bit " << just_hovered_track << "\n";
		controller->model->toggle_bit(just_hovered_row, just_hovered_track);
	}

	// reset pattern
	cairo_pattern_destroy(lochstreifen->highlight_bit_color);
	lochstreifen->highlight_bit_color = LOCHSTREIFEN_DEFAULT_HIGHLIGHT_BIT_COLOR;
	
	queue_draw_bit(just_hovered_row, just_hovered_track);

	return true;
}


void PaperTapeView::export_image() {
	std::cout << "Creating object:\n";
	PaperTapeExporter program(*controller->parent_window, lochstreifen);
	std::cout << "STARTING EXPORT\n";
	if(program.run_export()) {
		std::cout << "Export handled nicely\n";
	}
}

Glib::RefPtr<ActionGroup> PaperTapeView::get_color_action_group() {
	Glib::RefPtr<ActionGroup> group = ActionGroup::create("GtkPaperTapeViewColors");
	// iterate the Colors
	for(std::map<std::string,  Glib::RefPtr<PaperTapeColor> >::iterator i = colors.begin();
	    i != colors.end(); i++) {
		std::cout << "Adding actions for " << i->first << "\n";
		group->add(i->second);
		if(i->second->is_switchable())
			group->add(i->second->get_toggle_action());
	}

/*
	for(unsigned int i = 0; i < colors.size(); i++) {
		group->add(colors[i]->get_color_action());
		if(colors[i]->is_switchable())
			group->add(colors[i]->get_toggle_action());
	}
*/
	return group;
}

void PaperTapeView::make_colors() {
	// Make the color vector thing, for faster iteration
	add_color("outer_background",
		&lochstreifen->outer_background_color,
		"Select background color",
		"Color of the <b>background</b>",
		"Display background"
	);
	add_color("papertape_background",
		&lochstreifen->papertape_background_color,
		"Select color of papertape",
		"Color of the <b>papertape</b>",
		"Display papertape sheet"
	);
	add_color("one_code_hole",
		&lochstreifen->one_code_hole_color,
		"Select color of punched holes (logical 1)",
		"Color of <b>punched holes</b>\n(logical 1)",
		"Display punched code holes"
	);
	add_color("zero_code_hole",
		&lochstreifen->zero_code_hole_color,
		"Select color of zero holes (logical 0)",
		"Color of <b>zero holes</b>\n(logical 0)",
		"Display zero code holes"
	);
	add_color("feed_hole",
		&lochstreifen->feed_hole_color,
		"Select color of feed holes",
		"Color of <b>feed holes</b>",
		"Display feed holes"
	);
	add_color("highlight_region",
		&lochstreifen->highlight_region_color,
		"Select color of highlighted regions (selections)",
		"Color of <b>selections</b>"
	);
	add_color("highlight_row",
		&lochstreifen->highlight_row_color,
		"Select color of highlighted byte (hovered byte)",
		"Color of <b>hovered bytes</b>"
	);
	add_color("highlight_bit",
		&lochstreifen->highlight_bit_color,
		"Select color of highlighted bit (hovered bit)",
		"Color of <b>hovered bit</b>"
	);
}

void PaperTapeView::add_color(const Glib::ustring& name, cairo_pattern_t** target, const Glib::ustring& dialog_title,
   const Glib::ustring& menu_label, const Glib::ustring& toggle_label) {
	colors[name] = PaperTapeColor::create(this, target,
		name, dialog_title, menu_label, toggle_label);
	colors[name]->signal_color_changed().connect(sigc::mem_fun(*this, &PaperTapeView::on_color_change));
}

void PaperTapeView::on_color_change(PaperTapeColor* name) {
	std::cout << "PaperTapeView: Got a changed color!\n";
	queue_draw();
}
