#ifndef __PAPER_TAPE_VIEW__
#define __PAPER_TAPE_VIEW__

#include <gtkmm/drawingarea.h>
//#include <gtkmm/checkmenuitem.h>
//#include <gtkmm/menuitem.h>
//#include <gtkmm/sizegroup.h>
//#include <gtkmm/actiongroup.h>
#include <map>

namespace Gtk
{ class PaperTapeView; }
#include "view.h"
#include "../lochstreifen.h"
#include "color.h"
#include "zoom.h"

namespace PaperTape {

/**
 * The transformation action which shall be performed, like scaling
 * (zooming), rotation, mirroring.
 * @see Lochstreifen::transform
 **/
enum PaperTapeTransformation {
	ZOOM_IN,
	ZOOM_OUT,
	ZOOM_REAL,
	SET_ZOOM,
	ROTATE_CLOCKWISE,
	ROTATE_ANTICLOCKWISE,
	MIRROR_HOR,
	MIRROR_VER,
};

/**
 * The PaperTapeView is the base class for rendering of the LOCHSTREIFEN in GTK+.
 * It's supposed to be abstract baseclass for concrete implementations like
 * PaperTapeIOView, PaperTapeEditor, PaperTapeScrollbar,...
 *
 * In the PaperTape MVC concept, it acts as the view.
 *
 *
 **/
class View : public Gtk::DrawingArea {
public:
	std::map<std::string, Glib::RefPtr<PaperTape::Color> > colors;
	void add_color(
		const Glib::ustring& name,
		cairo_pattern_t** target,
		const Glib::ustring& dialog_title,
		const Glib::ustring& menu_label,
		const Glib::ustring& toggle_label = Glib::ustring()
	);
	
	PaperTape::Zoom* zoom; // must be link for compiling issues :/
	
private:	
	int current_width_scaling;
	
	/// the currently hovered values
	row_t just_hovered_row;
	track_t just_hovered_track;
	/// always valid, on_motion_motify will update them
	void make_colors();

protected:
public:
	/**
	 * This is the PaperTape controller object where this view is assigned
	 * to. It won't change in the PaperTapeView life time.
	 **/
	Chrome* const controller;
	
	/**
	 * Each View has it's own LOCHSTREIFEN object and therefore it's own
	 *  - colors
	 *  - transformation matrix
	 *  - Selections and Highlights
	 *  - croppings
	 * The DATA are shared between all views of one controller and hold by
	 * the model, PaperTapeFile.
	 *
	 * This must be a pointer due to the lochstreifen_ C interface functions,
	 * which all work with pointers. Every View shall have only one LOCHSTREIFEN
	 * object in it's lifetime.
	 *
	 **/
	LOCHSTREIFEN* lochstreifen;
	
	void on_data_change(byte_t* data, int data_len);
	void on_color_change(PaperTapeColor* name);
	void on_zoom_change();

public:
	//View(Chrome* controller);
	View(File* file);
	
	virtual bool on_expose_event(GdkEventExpose *event);
	virtual bool on_scroll_event (GdkEventScroll* event);
	virtual bool on_motion_notify_event(GdkEventMotion* event);
	virtual bool on_button_press_event(GdkEventButton* event);
	virtual bool on_button_release_event(GdkEventButton* event);
	
	/**
	 * DISPLAY GUI.
	 * Export the papertape with the current colors and size
	 * to the file named as argument.
	 * @returns if export successed.
	 **/
	void export_image();
	
	/**
	 * Zooming, Rotation, Mirroring, etc.
	 **/
	void transform(PaperTapeTransformation how) {
		std::cout << "dummy zoom.\n";
	}
	
	/**
	 * Queues drawing for these objects
	 **/
	inline void queue_draw_bit(row_t& row, track_t& track);
	inline void queue_draw_row(row_t&);
	void queue_draw_rect(cairo_rectangle_t& rect) {
		queue_draw_area(
			static_cast<int>( floor(rect.x) ),
			static_cast<int>( floor(rect.y) ),
			static_cast<int>( ceil (rect.width) ),
			static_cast<int>( ceil (rect.height) )
		);
	}

	/**
	 * Creates the color menuitems, connects signals, etc
	 * and returns actions for the color menu
	 **/
	Glib::RefPtr<Gtk::ActionGroup> get_color_action_group();
};

}; // namespace Gtk
#endif // __PAPER_TAPE_VIEW__
